;**************************************************************************
;*
;* FILE  flash.asm
;* Copyright (c) 2005, 2006, 2007 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: flash.asm,v 1.125 2007-09-10 21:54:56 tlr Exp $
;*
;* DESCRIPTION
;*   Program and verify flash memory.
;*
;* LAYOUT
;*   +---------------------------------+
;*   | Seg #0, Normal                  |
;*   | $0000                           |
;*   +---------------------------------+
;*   | Seg #1, RAM Read/Write Window   |
;*   | $4000                           |
;*   +---------------------------------+
;*   | Seg #2, Flash Read/Write Window |
;*   | $8000   (Possiby ROM)           |
;*   +---------------------------------+
;*   | Seg #3, Normal                  |
;*   | $c000                           |
;*   +---------------------------------+
;*
;******
	processor 6502


	MAC	sac
	dc.b	$32,{1}
	ENDM
	MAC	sir
	dc.b	$42,{1}
	ENDM
	MAC	bra
_TMPBRA	set	{1}-2-.
	if	_TMPBRA < -128 || _TMPBRA > 127
	ERR 	"Branch to long!"
	endif
	dc.b	$12,_TMPBRA
	ENDM

DEBUG_OUT	equ	1

; buffer
BUFFER_START	equ	$020000
		

;flags
REFRESH_HEADER	equ	%00000001
REFRESH_FOOTER	equ	%00000010
REFRESH_MENU	equ	%00000100
REFRESH_HOOK	equ	%00001000
REFRESH_ALL	equ	%00001111

	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$a3
len_zp:
	ds.b	2
tmpptr_zp:
	ds.b	3
rawmode_zp:
	ds.b	1
tmp3_zp:
	ds.b	1
	
	org	$b0
fptr_zp:
	ds.b	3
eptr_zp:
	ds.b	3
bank_zp:	
	ds.b	1

	org	$bd
tmp_zp:
	ds.b	1
tmp2_zp:
	ds.b	1

	org	$f7
xtmp_zp:
	ds.b	1
ytmp_zp:	
	ds.b	1
ptr_zp:	
	ds.w	1
ptr2_zp:	
	ds.w	1
rptr_zp:
	ds.b	3


;**************************************************************************
;*
;* SECTION  startmarkers
;*   
;******
	seg	data
	org	code_end

	seg.u	bss
	org	data_end
				
	seg	code
	org	$0801
;**************************************************************************
;*
;* Basic line!
;*
;******
StartOfFile:
	dc.w	EndLine
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
EndLine:
	dc.w	0
	
;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
SysAddress:
; restore system state after decrunch or similar
RESTORE	equ	1
	include	"sys.asm"
	
; set device number to 8 if it is less than 8
	lda	#8
	cmp	$ba
	bcc	sa_skp1
	sta	$ba
sa_skp1:
	
; enable key repeat
	lda	#255
	sta	650

; if this is the first time, show the warning screen
sa_sm1:
	jsr	first_time_init
	lda	#$2c
	sta	sa_sm1
	
; fix screen
	jsr	init_video

; move all roms we need into ram
	jsr	mirror_roms

; first identify the flash
	jsr	identify_flash

; if this is the first time, set up some defaults and show the intro
sa_sm2:
	jsr	first_time_init2
	lda	#$2c
	sta	sa_sm2

; show menu
	jsr	cmd_go_menu1
	lda	#REFRESH_ALL
	sta	mnu_refresh
	jsr	menu

	jsr	unmirror_roms
	jmp	$a474

;**************************************************************************
;*
;* NAME  first_time_init
;*
;* DESCRIPTION
;*   Only called the first time the program is run.
;*
;******
first_time_init:
	jsr	show_warning
	rts

;**************************************************************************
;*
;* NAME  first_time_init2
;*
;* DESCRIPTION
;*   Called after flash identification.
;*   Only called the first time the program is run.
;*
;******
first_time_init2:
; set default sector the first time we are run
	lda	#DEFAULT_SECTOR_ADDR & $ff
	sta	fptr_zp
	lda	#[DEFAULT_SECTOR_ADDR>>8] & $ff
	sta	fptr_zp+1
	lda	#[DEFAULT_SECTOR_ADDR>>16]
	sta	fptr_zp+2
	jsr	find_sector_num
	sta	curr_sector

	jsr	print_header
	jsr	print_footer
	jsr	show_intro
	rts


; $1f2000/$1e0000
DEFAULT_SECTOR_ADDR	equ	$1f2000

;**************************************************************************
;*
;* NAME  init_video
;*
;* DESCRIPTION
;*   Only called the first time the program is run.
;*
;******
init_video:
	jsr	$ff5b
	lda	#6
	sta	$d020
	lda	#0
	sta	$d021
	lda	#1
	sta	646
	rts


;**************************************************************************
;*
;* NAME  print_header
;*
;* DESCRIPTION
;*   show the screen heading and footer
;*
;******
print_header:
	lda	#<header_msg
	ldy	#>header_msg
	jsr	print

	ldy	#0
	ldx	#22
	clc
	jsr	$fff0
	lda	#<footer_msg
	ldy	#>footer_msg
	jsr	print
	rts
header_msg:
	dc.b	147,"DTV PAL/HUMMER FLASH UTILITY V1.0  / TLR"
	ds.b	40,195
	dc.b	0

footer_msg:
	ds.b	40,195
	dc.b	"FLASH:                        DRIVE #  ",13
	dc.b	"LOW-MEM WP:                            "
	dc.b	0

;**************************************************************************
;*
;* NAME  print_footer
;*
;* DESCRIPTION
;*   Update footer status info
;*
;******
print_footer:
; preserve cursor position
	sec
	jsr	$fff0
	txa
	pha
	tya
	pha

; Print flash type
	ldy	#7
	ldx	#23
	clc
	jsr	$fff0
	ldx	#12
	jsr	print_type
	ldy	#20
	jsr	clear_upto	
; Print device num
	ldy	#37
	ldx	#23
	clc
	jsr	$fff0
	ldx	$ba
	lda	#0
	jsr	$bdcd	
	ldy	#39
	jsr	clear_upto	
; Print low-mem wp.
	ldy	#11
	ldx	#24
	clc
	jsr	$fff0
	lda	lowmem_wp
	jsr	print_on_off

; restore cursor position
	pla
	tay
	pla
	tax
	clc
	jmp	$fff0

print_on_off:
	cmp	#0
	bne	poo_ex1
	lda	#<off_msg
	ldy	#>off_msg
	jmp	print
poo_ex1:
	lda	#<on_msg
	ldy	#>on_msg
	jmp	print
on_msg:	
	dc.b	"ON ",0
off_msg:	
	dc.b	"OFF",0
	
;**************************************************************************
;*
;* NAME  clear_menu
;*
;* DESCRIPTION
;*   Clear the central screen area.
;*
;******
clear_menu:
; reset link table
	lda	$0288	;screen base
	ora	#$80
	tay
	lda	#0
	tax
cm_lp1:
	sty	$d9,x
	clc
	adc	#40
	bcc	cm_skp1
	iny
cm_skp1:
	inx
	cpx	#26
	bne	cm_lp1
	lda	#$ff
	sta	$d9,x

	ldx	#2
cm_lp2:
	jsr	$e9ff
	inx
	cpx	#22
	bne	cm_lp2

; put cursor at 0,2
	ldy	#0
	ldx	#2
	sty	$d3
	stx	$d6
	jmp	$e56c		; update screen ptr


	if	0
	clc
	jsr	$fff0

	ldx	#40*5
	lda	#$20
cm_lp2:
	sta	$0400+[40*2]-1,x
	sta	$0400+[40*7]-1,x
	sta	$0400+[40*12]-1,x
	sta	$0400+[40*17]-1,x
	dex
	bne	cm_lp2
	rts
	endif		
;**************************************************************************
;*
;* NAME  menu
;*
;* DESCRIPTION
;*   The menu code
;*
;******
menu:
	lda	#0
	sta	mnu_exit
; main loop
mnu_mainloop:
	lda	mnu_exit
	beq	mnu_skp1
	rts
mnu_skp1:
	lda	mnu_refresh
	and	#REFRESH_HEADER
	beq	mnu_skp2
	eor	mnu_refresh
	sta	mnu_refresh

	jsr	print_header

mnu_skp2:
	lda	mnu_refresh
	and	#REFRESH_FOOTER
	beq	mnu_skp3
	eor	mnu_refresh
	sta	mnu_refresh

	jsr	print_footer

mnu_skp3:
	lda	mnu_refresh
	and	#REFRESH_MENU
	beq	mnu_skp4
	eor	mnu_refresh
	sta	mnu_refresh

	jsr	clear_menu
	lda	mnu_active
	sta	ptr2_zp
	lda	mnu_active+1
	sta	ptr2_zp+1
	ldy	#0
	lda	(ptr2_zp),y
	sta	mnu_sm1+1
	iny
	lda	(ptr2_zp),y	
	sta	mnu_sm1+2
mnu_sm1:
	jsr	mnu_sm1

mnu_skp4:
	lda	mnu_refresh
	and	#REFRESH_HOOK
	beq	mnu_skp5
	eor	mnu_refresh
	sta	mnu_refresh
	
; call the menu refresh hook
	ldy	#2
	lda	(ptr2_zp),y
	sta	mnu_sm2+1
	iny
	lda	(ptr2_zp),y
	sta	mnu_sm2+2
mnu_sm2:
	jsr	mnu_sm2

mnu_skp5:
; restore pointers that might have been destroyed.
	lda	mnu_active
	sta	ptr2_zp
	lda	mnu_active+1
	sta	ptr2_zp+1
; wait for key, and jump to the appropriate routine
mnu_lp1:
	jsr	$ffe4
	beq	mnu_lp1
	tax
	ldy	#4
mnu_lp2:
	lda	(ptr2_zp),y
	beq	mnu_lp1		;end of list, not found !
	cmp	#$ff
	beq	mnu_skp6	;$ff, skip this entry.
	txa
	cmp	(ptr2_zp),y
	beq	mnu_found
mnu_skp6:
	iny
	iny
	iny
	bne	mnu_lp2
; if more that 256, loop back to read keys (should never happen!)
	bra	mnu_lp1

mnu_found:
	iny
	lda	(ptr2_zp),y
	sta	mnu_sm3+1
	iny
	lda	(ptr2_zp),y
	sta	mnu_sm3+2
mnu_sm3:
	jsr	mnu_sm3
	jmp	mnu_mainloop

mnu_exit:
	dc.b	0
mnu_refresh:
	dc.b	0
mnu_active:
	dc.w	menu1


;**************************************************************************
;*
;* NAME  menu1
;*
;* DESCRIPTION
;*   Default menu with range operations
;*
;******
cmd_go_menu1:
	lda	#REFRESH_MENU | REFRESH_HOOK
	sta	mnu_refresh
	lda	#<menu1
	sta	mnu_active
	lda	#>menu1
	sta	mnu_active+1
	rts

menu1:
	dc.w	menu1_print
	dc.w	menu1_refresh
	dc.b	"X"
	dc.w	cmd_exit
	dc.b	"P"
	dc.w	cmd_program_range
	dc.b	"V"
	dc.w	cmd_verify_range
	dc.b	"D"
	dc.w	cmd_dump_range
	dc.b	"E"
	dc.w	cmd_erase_range
	dc.b	"C"
	dc.w	cmd_empty_test_range
	dc.b	"M"
	dc.w	cmd_show_map
	dc.b	"W"
	dc.w	cmd_toggle_wp
	dc.b	"I"
	dc.w	cmd_reidentify
	dc.b	"L"
	dc.w	cmd_load_data
	dc.b	"S"
	dc.w	cmd_save_data
	dc.b	"$"
	dc.w	cmd_directory
	dc.b	"8"
	dc.w	cmd_set_drive8
	dc.b	"9"
	dc.w	cmd_set_drive9
	dc.b	"A"
	dc.w	cmd_go_menu2
	dc.b	"H"
	dc.w	cmd_help_text
	dc.b	0

menu1_msg:
	dc.b	13
	dc.b	"P     - PROGRAM RANGE FROM BUFFER",13
	dc.b	"V     - VERIFY RANGE AGAINST BUFFER",13
	dc.b	"D     - DUMP RANGE TO BUFFER",13
	dc.b	"E     - ERASE RANGE",13
	dc.b	"M     - SHOW MAP OF FLASH",13
	dc.b	"C     - CHECK IF RANGE IS EMPTY",13
	dc.b	"W     - TOGGLE LOW-MEM WP",13
	dc.b	"I     - REIDENTIFY FLASH",13
	dc.b	"L/S   - LOAD/SAVE DATA",13
	dc.b	"$     - DIRECTORY",13
	dc.b	"8/9   - SELECT DRIVE",13
	dc.b	"A     - ADDITIONAL OPERATIONS",13
	dc.b	"H     - HELP",13
	dc.b	"X     - EXIT",13
	dc.b	13
	dc.b	0

menu1_print:
	lda	#<menu1_msg
	ldy	#>menu1_msg
	jmp	print

menu1_refresh:
	rts


;**************************************************************************
;*
;* NAME  menu2
;*
;* DESCRIPTION
;*   Sector and lowlevel operations menu
;*
;******
cmd_go_menu2:
	lda	#REFRESH_MENU | REFRESH_HOOK
	sta	mnu_refresh
	lda	#<menu2
	sta	mnu_active
	lda	#>menu2
	sta	mnu_active+1

	lda	#"K"
	ldx	have_sector_lockdown
	bne	cgm2_skp1
	lda	#$ff
cgm2_skp1:
	sta	m2_sm1
	rts

menu2:
	dc.w	menu2_print
	dc.w	menu2_refresh
	dc.b	"X"
	dc.w	cmd_exit
	dc.b	"R"
	dc.w	cmd_read_raw_flash_id
	dc.b	133		; F1
	dc.w	cmd_inc_sector
	dc.b	137		; F2
	dc.w	cmd_dec_sector
m2_sm1:
	dc.b	"K"
	dc.w	cmd_lockdown_sector
	dc.b	"P"
	dc.w	cmd_program_sector
	dc.b	"B"
	dc.w	cmd_program_byte
	dc.b	"V"
	dc.w	cmd_verify_sector
	dc.b	"D"
	dc.w	cmd_dump_sector
	dc.b	"E"
	dc.w	cmd_erase_sector
	dc.b	"C"
	dc.w	cmd_empty_test_sector
	dc.b	"M"
	dc.w	cmd_show_map
	dc.b	"W"
	dc.w	cmd_toggle_wp
	dc.b	"I"
	dc.w	cmd_reidentify
	dc.b	"L"
	dc.w	cmd_load_data
	dc.b	"S"
	dc.w	cmd_save_data
	dc.b	"$"
	dc.w	cmd_directory
	dc.b	"8"
	dc.w	cmd_set_drive8
	dc.b	"9"
	dc.w	cmd_set_drive9
	dc.b	"A"
	dc.w	cmd_go_menu1
	dc.b	"H"
	dc.w	cmd_help_text
	dc.b	0

menu2_1_msg:
	dc.b	13
	dc.b	"R     - READ RAW FLASH IDENTITY",13
	dc.b	"F1/F2 - INC/DEC SECTOR ..=......-......",13
	dc.b	0
menu2_2_lockdown_msg:
	dc.b	"K     - LOCKDOWN SECTOR",13
menu2_2_msg:
	dc.b	"B     - PROGRAM SINGLE BYTE",13
	dc.b	"P     - PROGRAM SECTOR FROM BUFFER",13
	dc.b	"V     - VERIFY SECTOR AGAINST BUFFER",13
	dc.b	"D     - DUMP SECTOR TO BUFFER",13
	dc.b	"E     - ERASE SECTOR",13
	dc.b	"M     - SHOW MAP OF FLASH",13
	dc.b	"C     - CHECK IF EMPTY",13
	dc.b	"W     - TOGGLE LOW-MEM WP",13
	dc.b	"L/S   - LOAD/SAVE DATA",13
	dc.b	"$     - DIRECTORY",13
	dc.b	"8/9   - SELECT DRIVE",13
	dc.b	"A     - ADDITIONAL OPERATIONS",13
	dc.b	"H     - HELP",13
	dc.b	"X     - EXIT",13
	dc.b	0
	
menu2_print:
	lda	#<menu2_1_msg
	ldy	#>menu2_1_msg
	jsr	print
	lda	have_sector_lockdown
	beq	m2p_skp1
	lda	#<menu2_2_lockdown_msg
	ldy	#>menu2_2_lockdown_msg
	jmp	print
m2p_skp1:
	lda	#<menu2_2_msg
	ldy	#>menu2_2_msg
	jmp	print

menu2_refresh:	
	ldy	#23
	ldx	#4
	clc
	jsr	$fff0
	
	lda	curr_sector
	jsr	print_hex

	lda	#"="
	jsr	$ffd2
	lda	curr_sector
	jsr	set_sector_addr
	jsr	print_fptr
	lda	#"-"
	jsr	$ffd2
	lda	curr_sector
	clc
	adc	#1
	jsr	set_sector_addr
	jsr	print_fptr
	
	ldy	#0
	ldx	#20
	clc
	jsr	$fff0
	rts
	
sa_wait_key:	
	lda	#<anykey_msg
	ldy	#>anykey_msg
	jsr	print
sa_wait_key2:
sa_lp4:
	jsr	$ffe4
	beq	sa_lp4
sa_exit_refresh:
	lda	#REFRESH_ALL
	sta	mnu_refresh
	rts

anykey_msg:
	dc.b	13,"PRESS ANY KEY.",13,0

;**************************************************************************
;*
;* NAME  cmd_exit
;*
;* DESCRIPTION
;*   set exit flag
;*
;******
cmd_exit:
	jsr	ask_sure
	bcc	cex_ex1
	
	lda	#<exit_msg
	ldy	#>exit_msg
	jsr	print
	lda	#1
	sta	mnu_exit
cex_ex1:
	lda	#REFRESH_MENU | REFRESH_HOOK
	sta	mnu_refresh
	rts

exit_msg:
	dc.b	147,"EXITING...",13,0

;**************************************************************************
;*
;* NAME  cmd_inc_sector, cmd_dec_sector
;*
;* DESCRIPTION
;*   increase/decrease sector number
;*
;******
cmd_inc_sector:
	lda	#REFRESH_HOOK
	sta	mnu_refresh
	inc	curr_sector
	lda	curr_sector
	cmp	num_sectors
	beq	cis_skp1
	rts
cis_skp1:
	lda	#0
	sta	curr_sector
	rts
	
cmd_dec_sector:
	lda	#REFRESH_HOOK
	sta	mnu_refresh
	lda	curr_sector
	bne	cds_skp1
	lda	num_sectors
	sta	curr_sector
cds_skp1:
	dec	curr_sector
	rts

;**************************************************************************
;*
;* NAME  cmd_set_drive8, cmd_set_drive9
;*
;* DESCRIPTION
;*   set drive numbers.
;*
;******
cmd_set_drive8:
	lda	#8
	dc.b	$2c
cmd_set_drive9:
	lda	#9
	sta	$ba
	lda	#REFRESH_FOOTER
	sta	mnu_refresh
	rts

;**************************************************************************
;*
;* NAME  cmd_toggle_wp
;*
;* DESCRIPTION
;*   Toggle the soft write protect
;*
;******
cmd_toggle_wp:
	lda	lowmem_wp
	beq	ctw_skp1

	lda	#<toggle_wp_msg
	ldy	#>toggle_wp_msg
	jsr	print
	jsr	ask_sure
	bcc	ctw_ex1
		
	lda	#0
	sta	lowmem_wp
ctw_ex1:
	lda	#REFRESH_ALL
	sta	mnu_refresh
	rts
	
ctw_skp1:
	lda	#1
	sta	lowmem_wp
	lda	#REFRESH_ALL
	sta	mnu_refresh
	rts

toggle_wp_msg:
	dc.b	147,"DISABLE LOW-MEM WP...",13,13,0
		
;**************************************************************************
;*
;* NAME  cmd_reidentify
;*
;* DESCRIPTION
;*   Reidentify flash type
;*
;******
cmd_reidentify:
	lda	curr_sector
	jsr	set_sector_addr

; must not destroy fptr_zp
	jsr	identify_flash

; used here
	jsr	find_sector_num
	sta	curr_sector

	lda	#REFRESH_FOOTER | REFRESH_MENU | REFRESH_HOOK
	sta	mnu_refresh
	rts

;**************************************************************************
;*
;* NAME  cmd_help_text
;*
;* DESCRIPTION
;*   display help text.
;*
;******
cmd_help_text:
	ldx	#<helptext
	ldy	#>helptext
	jsr	show_help
	jmp	sa_exit_refresh

;**************************************************************************
;*
;* NAME  input_range
;*
;* DESCRIPTION
;*   Input ranges.  Handle memory of the last one.
;*   C=1 means error. (this does not check for valid ranges though.)
;*
;******
input_range:
	lda	#<start_addr_msg
	ldy	#>start_addr_msg
	jsr	print
	lda	range_start
	ldy	range_start+1
	ldx	range_start+2
	jsr	input_triplet_sel
	php
	sta	range_start
	sty	range_start+1
	stx	range_start+2
	jsr	print_cr
	plp
	bcs	inr_fl1

	lda	#<end_addr_msg
	ldy	#>end_addr_msg
	jsr	print
	lda	range_end
	ldy	range_end+1
	ldx	range_end+2
	jsr	input_triplet_sel
	php
	sta	range_end
	sty	range_end+1
	stx	range_end+2
	jsr	print_cr
	plp
inr_fl1:
	rts

input_triplet_sel:
	cpx	#$ff
	beq	ir_skp1
	jmp	input_triplet_pre
ir_skp1:
	jmp	input_triplet

set_range_fptr_eptr:
	ldx	#2
srfe_lp1:
	lda	range_start,x
	sta	fptr_zp,x
	lda	range_end,x
	sta	eptr_zp,x
	dex
	bpl	srfe_lp1
	rts
	
start_addr_msg:
	dc.b	"START $------",157,157,157,157,157,157,0
end_addr_msg:
	dc.b	"END   $------",157,157,157,157,157,157,0

range_start:
	dc.b	-1,-1,-1
range_end:
	dc.b	-1,-1,-1

;**************************************************************************
;*
;* NAME  cmd_dump_range
;*
;* DESCRIPTION
;*   Dump a range of the flash.
;*
;******
cmd_dump_range:
	lda	#<dump_range_msg
	ldy	#>dump_range_msg
	jsr	print
	jsr	input_range
	bcs	cdra_fl1

	jsr	map_flash

	jsr	set_range_fptr_eptr

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2
	
	jsr	dump_range
	jsr	unmap_flash
	jsr	print_err
cdra_fl1:
	jmp	sa_wait_key

dump_range_msg:
	dc.b	147,"DUMP RANGE...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_verify_range
;*
;* DESCRIPTION
;*   Verify a range of the flash.
;*
;******
cmd_verify_range:
	lda	#<verify_range_msg
	ldy	#>verify_range_msg
	jsr	print

	jsr	input_range
	bcs	cvra_fl1
	jsr	map_flash

	jsr	set_range_fptr_eptr

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2

	jsr	verify_range
	jsr	unmap_flash

	jsr	print_err
cvra_fl1:
	jmp	sa_wait_key

verify_range_msg:
	dc.b	147,"VERIFY RANGE...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_program_range
;*
;* DESCRIPTION
;*   Program a range of the flash.
;*
;******
cmd_program_range:
	lda	#<program_range_msg
	ldy	#>program_range_msg
	jsr	print

	jsr	input_range
	bcs	cpr_fl1

	jsr	ask_sure
	bcc	cpr_ex1

	jsr	map_flash

	jsr	set_range_fptr_eptr

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2

	jsr	program_range
	jsr	unmap_flash
	jsr	print_err

cpr_fl1:
	jmp	sa_wait_key
	
cpr_ex1:
	jmp	sa_exit_refresh

program_range_msg:
	dc.b	147,"PROGRAM RANGE...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_erase_range
;*
;* DESCRIPTION
;*   Erase a range of the flash.
;*
;******
cmd_erase_range:
	lda	#<erase_range_msg
	ldy	#>erase_range_msg
	jsr	print

	jsr	input_range
	bcs	cer_fl1
	
	jsr	ask_sure
	bcc	cer_ex1

	jsr	map_flash
	jsr	set_range_fptr_eptr
	jsr	erase_range
	jsr	unmap_flash
	jsr	print_err

cer_fl1:	
	jmp	sa_wait_key
cer_ex1:
	jmp	sa_exit_refresh

erase_range_msg:
	dc.b	147,"ERASE RANGE...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_empty_test_range
;*
;* DESCRIPTION
;*   read range to check if it is empty
;*
;******
cmd_empty_test_range:
	lda	#<empty_test_msg
	ldy	#>empty_test_msg
	jsr	print

	jsr	input_range
	bcs	cetr_fl1

	jsr	map_flash
	jsr	set_range_fptr_eptr
	jsr	check_empty
	jsr	unmap_flash
	cmp	#$ff
	bne	cetr_notempty

	lda	#<empty_msg
	ldy	#>empty_msg
	jsr	print
cetr_fl1:
	jmp	sa_wait_key
	
cetr_notempty:	
	lda	#<not_empty_msg
	ldy	#>not_empty_msg
	jsr	print
	jmp	sa_wait_key

;**************************************************************************
;*
;* NAME  cmd_show_map
;*
;* DESCRIPTION
;*   Show a map of flash usage.
;*
;******
cmd_show_map:
	jsr	clear_menu
	lda	646
	pha
	lda	#14
	sta	646
	lda	#<show_map_msg
	ldy	#>show_map_msg
	jsr	print
	pla
	sta	646
		
	jsr	map_flash
	
; setup fptr_zp=$000000, eptr_zp=$001000
	ldy	#$10
	sty	eptr_zp+1
	ldy	#$00
	sty	fptr_zp
	sty	fptr_zp+1
	sty	fptr_zp+2
	sty	eptr_zp
	sty	eptr_zp+2
	sty	rptr_zp
	sty	rptr_zp+1
csp_lp1:	
	jsr	find_sector_num
	and	#SECTOR_EVEN
	sta	tmp_zp
	tax
	jsr	check_empty
	cmp	#$ff
	bne	csp_skp3
	lda	tmp_zp
	ora	#SECTOR_EMPTY
	tax
csp_skp3:

	lda	rptr_zp
	ldy	rptr_zp+1
	jsr	print_stat

	inc	rptr_zp
	bne	csp_skp1
	inc	rptr_zp+1
csp_skp1:

	lda	eptr_zp+1
	sta	fptr_zp+1
	lda	eptr_zp+2
	sta	fptr_zp+2
	
	lda	eptr_zp+1
	clc
	adc	#$10
	sta	eptr_zp+1
	bcc	csp_skp2
	inc	eptr_zp+2
csp_skp2:

	lda	fptr_zp+2	; fptr_zp=$200000?
	cmp	#$20
	bne	csp_lp1		; no, continue.
		
	jsr	unmap_flash

; exit and wait for key
	jmp	sa_wait_key2


; Acc=LSB, Y=MSB
print_stat:
	stx	xtmp_zp
	sta	tmp_zp
	and	#$0f
	asl
	tax
	lda	basetab,x	
	sta	ptr_zp
	sta	ptr2_zp
	lda	basetab+1,x	
	sta	ptr_zp+1
	clc
	adc	#$d8-$04
	sta	ptr2_zp+1
	tya
	lsr
	ror	tmp_zp
	lsr
	ror	tmp_zp
	lsr
	ror	tmp_zp
	lsr
	ror	tmp_zp
	ldy	tmp_zp
	
	ldx	xtmp_zp
	lda	chartab,x
	sta	(ptr_zp),y
	lda	colortab,x
	sta	(ptr2_zp),y
	rts

			
SB	equ	$0400+40*4+4
basetab:
	dc.w	SB+40*0,SB+40*1,SB+40*2,SB+40*3
	dc.w	SB+40*4,SB+40*5,SB+40*6,SB+40*7
	dc.w	SB+40*8,SB+40*9,SB+40*10,SB+40*11
	dc.w	SB+40*12,SB+40*13,SB+40*14,SB+40*15

chartab:
	dc.b	81,81,".","."
colortab:
	dc.b	5,13,5,13

SECTOR_EVEN	equ	%00000001
SECTOR_EMPTY	equ	%00000010
SECTOR_LOCKED	equ	%00000100

	seg.u	bss
statusflags:
	ds.b	512
	
	seg	code
;**************************************************************************
;*
;* NAME  cmd_program_byte
;*
;* DESCRIPTION
;*   read status from the flash and present to screen.
;*
;******
cmd_program_byte:
	lda	#<progbyte_msg
	ldy	#>progbyte_msg
	jsr	print

	lda	#<offs_msg
	ldy	#>offs_msg
	jsr	print

	lda	curr_sector
	jsr	set_sector_addr
	lda	fptr_zp
	ldy	fptr_zp+1
	ldx	fptr_zp+2
	jsr	input_triplet_pre
	sta	fptr_zp
	sty	fptr_zp+1
	stx	fptr_zp+2
	jsr	print_cr
	
	jsr	map_flash
	ldy	#0
	jsr	read_fbyte
	sta	tmp2_zp
	jsr	unmap_flash	

	lda	#<value_msg
	ldy	#>value_msg
	jsr	print

	lda	tmp2_zp
	jsr	print_hex
	lda	#<value2_msg
	ldy	#>value2_msg
	jsr	print
	lda	tmp2_zp
	jsr	input_byte_pre
	sta	tmp3_zp
	jsr	print_cr

; if already the correct value, skip out.
	lda	tmp2_zp
	cmp	tmp3_zp
	beq	cdp_ex1
	
; if we demand 0-bits be set to one, report as impossible
	lda	tmp2_zp
	and	tmp3_zp
	cmp	tmp3_zp
	bne	cpb_imp
	
; program the byte
	jsr	map_flash
	lda	tmp3_zp
	jsr	program_byte
	jsr	unmap_flash

	jsr	print_err
; programming went ok, exit.
	jmp	sa_wait_key

cdp_ex1:
	lda	#<skipped_msg
	ldy	#>skipped_msg
	jsr	print
	jmp	sa_wait_key

cpb_imp:
	lda	#<impossible_msg
	ldy	#>impossible_msg
	jsr	print
	jmp	sa_wait_key

progbyte_msg:
	dc.b	147,"PROGRAM BYTE...",13,13,0
value_msg:
	dc.b	"VALUE ($",0
value2_msg:
	dc.b	") $",0
offs_msg:
	dc.b	"OFFSET $",0
impossible_msg:
	dc.b	"IMPOSSIBLE!",13,0
skipped_msg:
	dc.b	"SKIPPED.",13,0
	
;**************************************************************************
;*
;* NAME  cmd_load_data
;*
;* DESCRIPTION
;*   read status from the flash and present to screen.
;*
;******
cmd_load_data:
	lda	#<loadfile_msg
	ldy	#>loadfile_msg
	jsr	print

cld_lp2:
	lda	#<filename_msg
	ldy	#>filename_msg
	jsr	print
	jsr	input_filename
	jsr	print_cr

;check length, exit if = 0
	lda	$b7
	bne	cld_skp4
	jmp	sa_exit_refresh

cld_skp4:
;check if directory
	ldy	#0
	lda	($bb),y
	cmp	#"$"
	bne	cld_skp3

	jsr	print_cr
	jsr	show_dir
	jsr	print_cr
	jmp	cld_lp2

cld_skp3:
	lda	#<offset_msg
	ldy	#>offset_msg
	jsr	print

	lda	#0
	tax
	tay
	jsr	input_triplet_pre
	sta	fptr_zp
	sty	fptr_zp+1
	stx	fptr_zp+2
	jsr	print_cr
	
	lda	#<rawdata_msg
	ldy	#>rawdata_msg
	jsr	print
	lda	#"N"
	jsr	input_y_n_pre
	sta	rawmode_zp
	jsr	print_cr


; add the start address of the buffer
	lda	fptr_zp
	sta	rptr_zp
	lda	fptr_zp+1
	sta	rptr_zp+1
	lda	fptr_zp+2
	clc
	adc	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2	

	
; use last accessed device
	lda	#1
	ldx	$ba
	ldy	#0		; read
	jsr	$ffba

; Print "SEARCHING FOR <name>"  (skipping the direct mode check at $f5af)
 	jsr	$f5b3		

; open the file.
	jsr	$ffc0
	lda	$90
	and	#%10000011
	bne	cld_fl3
	
; make it default input
	ldx	#1
	jsr	$ffc6

	lda	rawmode_zp
	cmp	#"Y"
	beq	cld_skp2
; skip the load address
	jsr	$ffe4
	jsr	$ffe4
cld_skp2:
		
	lda	#<loading_from_msg
	ldy	#>loading_from_msg
	jsr	print
	jsr	print_fptr
	lda	#<to_msg
	ldy	#>to_msg
	jsr	print

; load data
cld_lp1:
	jsr	$ffe4
	ldy	#0
	jsr	write_rbyte

	lda	$90		; check status, exit if fail
	and	#%10000011
	bne	cld_fl1

	inc	rptr_zp
	bne	cld_skp1

	ldy	#25
	jsr	dbg_printrptr_full
	
	inc	rptr_zp+1
	bne	cld_skp1
	inc	rptr_zp+2
cld_skp1:
	
; check for end of file
	bit	$90
	bvc	cld_lp1		;if not finished... loop!

	lda	#0
cld_fl1:	
	php
; restore default input
	jsr	$ffcc
; and close file
	lda	#1
	jsr	$ffc3
	
	jsr	unmap_ram
	
	ldy	#25
	jsr	dbg_printrptr_full
	jsr	dbg_printcr

; pop the status and check
	plp
	bne	cld_fl2

	lda	#<ok_msg
	ldy	#>ok_msg
cld_ex1:
	jsr	print
	jmp	sa_wait_key

cld_fl3:
; restore default input
	jsr	$ffcc
; close file
	lda	#1
	jsr	$ffc3		
	jsr	print_cr
cld_fl2:
	lda	#<dfailed_msg
	ldy	#>dfailed_msg
	jmp	cld_ex1

loadfile_msg:
	dc.b	147,"LOAD FILE TO BUFFER...",13,13,0
filename_msg:
	dc.b	"FILENAME:  ",0
offset_msg:
	dc.b	"START OFFS $",0
endoffset_msg:
	dc.b	"END OFFS   $",0
rawdata_msg:
	dc.b	"RAW DATA?  ",0
loading_from_msg:
	dc.b	13,"LOADING FROM $",0
to_msg:
	dc.b	" TO $------",0
dfailed_msg:
	dc.b	"FAILED!",13,0

;**************************************************************************
;*
;* NAME  cmd_save_data
;*
;* DESCRIPTION
;*   read status from the flash and present to screen.
;*
;******
cmd_save_data:
	lda	#<savefile_msg
	ldy	#>savefile_msg
	jsr	print

csd_lp2:
	lda	#<filename_msg
	ldy	#>filename_msg
	jsr	print
	jsr	input_filename
	jsr	print_cr

;check length, exit if = 0
	lda	$b7
	bne	csd_skp4
	jmp	sa_exit_refresh

csd_skp4:
;check if directory
	ldy	#0
	lda	($bb),y
	cmp	#"$"
	bne	csd_skp3

	jsr	print_cr
	jsr	show_dir
	jsr	print_cr
	jmp	csd_lp2

csd_skp3:
	
	lda	#<offset_msg
	ldy	#>offset_msg
	jsr	print
	lda	#0
	tax
	tay
	jsr	input_triplet_pre
	sta	fptr_zp
	sty	fptr_zp+1
	stx	fptr_zp+2
	jsr	print_cr

	lda	#<endoffset_msg
	ldy	#>endoffset_msg
	jsr	print
	lda	fptr_zp
	ldy	fptr_zp+1
	ldx	fptr_zp+2
	jsr	input_triplet_pre
	sta	eptr_zp
	sty	eptr_zp+1
	stx	eptr_zp+2
	jsr	print_cr
	
	lda	#<rawdata_msg
	ldy	#>rawdata_msg
	jsr	print
	lda	#"N"
	jsr	input_y_n_pre
	sta	rawmode_zp
	jsr	print_cr

; add the start address of the buffer
	lda	fptr_zp
	sta	rptr_zp
	lda	fptr_zp+1
	sta	rptr_zp+1
	lda	fptr_zp+2
	clc
	adc	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2	

	lda	eptr_zp+2
	clc
	adc	#(BUFFER_START>>16) & $ff
	sta	eptr_zp+2
	
; Print "SAVING <name>"  (skipping the direct mode check at $f68f)
 	jsr	$f693

; use last accessed device
	lda	#1
	ldx	$ba
	ldy	#1
	jsr	$ffba

	jsr	$ffc0
	lda	$90
	and	#%10000011
	bne	csd_fl1

; make it default output
	ldx	#1
	jsr	$ffc9

	lda	rawmode_zp
	cmp	#"Y"
	beq	csd_skp2
; the load address is just the low 16-bits of the start offset
 	lda	fptr_zp
	jsr	$ffd2
 	lda	fptr_zp+1
	jsr	$ffd2
csd_skp2:
	
	ldy	#0
csd_lp1:
	jsr	read_rbyte
	jsr	$ffd2

	lda	$90		; check status, exit if fail
	and	#%10000011
	bne	csd_fl1	

	inc	rptr_zp
	bne	csd_skp1
	inc	rptr_zp+1
	bne	csd_skp1
	inc	rptr_zp+2
csd_skp1:
	lda	eptr_zp
	cmp	rptr_zp
	bne	csd_lp1
	lda	eptr_zp+1
	cmp	rptr_zp+1
	bne	csd_lp1
	lda	eptr_zp+2
	cmp	rptr_zp+2
	bne	csd_lp1
	
	lda	#0
csd_fl1:
	php
; restore default output
	jsr	$ffcc
; and close file
	lda	#1
	jsr	$ffc3

	jsr	unmap_ram

	jsr	print_cr
	
; pop the status and check
	plp
	bne	csd_fl2

	lda	#<ok_msg
	ldy	#>ok_msg
csd_ex1:
	jsr	print
	jmp	sa_wait_key

csd_fl2:
	lda	#<dfailed_msg
	ldy	#>dfailed_msg
	jmp	csd_ex1

savefile_msg:
	dc.b	147,"SAVE FILE FROM BUFFER...",13,13,0

	
cmd_directory:
	lda	#147
	jsr	$ffd2
	jsr	show_dir
	jmp	sa_wait_key

;**************************************************************************
;*
;* NAME  show_dir
;*
;* DESCRIPTION
;*   show directory listing
;*
;******
show_dir:

; initialize and open
	lda	#1
	ldx	#<dir_msg
	ldy	#>dir_msg
	jsr	$ffbd	;SETNAM
	lda	#1
	ldx	$ba
	ldy	#$60
	jsr	$ffba	;SETLFS
	jsr	$ffc0	;OPEN
	lda	$90
	and	#%10000011
	bne	sdr_fl1
	lda	$ba
	jsr	$ffb4	;TALK
	lda	$b9
	jsr	$ff96	;TKSA

	ldy	#$03
sdr_lp1:
	sty	sdr_tmp
sdr_lp2:
	jsr	$ffa5	;ACPTR
	sta	sdr_linenum
	jsr	$ffa5	;ACPTR
	sta	sdr_linenum+1
	ldy	$90
	bne	sdr_ex1
	dec	sdr_tmp
	bne	sdr_lp2
	jsr	sdr_outputdecimal
	lda	#" "
	jsr	$ffd2	;CHROUT
sdr_lp3:
	jsr	$ffa5	;ACPTR
	ldx	$90
	bne	sdr_ex1
	cmp	#0
	beq	sdr_skp1
	jsr	$ffd2	;CHROUT
	jmp	sdr_lp3
sdr_skp1:
	lda	#13
	jsr	$ffd2	;CHROUT
	ldy	#$02
sdr_lp4:
	jsr	$ffe1	;STOP
	bne	sdr_lp1

sdr_ex1:

;*
;* exit, clean up
;*
	jsr	$ffab	;UNTLK
	lda	#1
	jsr	$ffc3	;CLOSE
sdr_fl1:
	jsr	$ffe7	;CLALL
	clc
	rts

sdr_linenum:
	dc.w	0
sdr_tmp:
	dc.b	0

sdr_outputdecimal:
	ldx	sdr_linenum
	lda	sdr_linenum+1
	jsr	$bdcd	;Output decimal number
	rts

dir_msg:
	dc.b	"$"
			
;**************************************************************************
;*
;* NAME  cmd_read_raw_flash_id
;*
;* DESCRIPTION
;*   read status from the flash and present to screen.
;*
;******
cmd_read_raw_flash_id:
	lda	#<start_msg
	ldy	#>start_msg
	jsr	print

	jsr	map_flash

; enter product id mode
	jsr	enter_product_id

; read IDs
	ldy	#0
crot_lp1:
	lda	FLASH_WINDOW,y
	sta	tmp_id_buffer,y
	lda	FLASH_WINDOW+$0100,y
	sta	tmp_id_buffer+$0100,y
	iny
	bne	crot_lp1
	
; exit product id mode
	jsr	exit_product_id

; enter cfi query mode
	jsr	enter_cfi_query_mode

; read cfi data
	ldy	#0
crot_lp2:
	lda	FLASH_WINDOW,y
	sta	tmp_cfi_buffer,y
	lda	FLASH_WINDOW+$0100,y
	sta	tmp_cfi_buffer+$0100,y
	iny
	bne	crot_lp2
	
; exit product id mode
	jsr	exit_product_id

	jsr	unmap_flash

; ok, data read.  Now make sense of it.
	lda	#$00
	sta	ptr2_zp
	lda	#>tmp_id_buffer
	sta	ptr2_zp+1

	lda	#<id0000_msg
	ldy	#>id0000_msg
	ldx	#16
	jsr	dump_buffer

	inc	ptr2_zp+1
	lda	#<id0100_msg
	ldy	#>id0100_msg
	ldx	#32
	jsr	dump_buffer

; cfi_mode
	lda	#$00
	sta	ptr2_zp
	lda	#>tmp_cfi_buffer
	sta	ptr2_zp+1

	lda	#<cfi0000_msg
	ldy	#>cfi0000_msg
	ldx	#192
	jsr	dump_buffer


	lda	#<copy_to_buffer_msg
	ldy	#>copy_to_buffer_msg
	jsr	print
	lda	#"N"
	jsr	input_y_n_pre
	pha
	jsr	print_cr
	pla
	cmp	#"Y"
	bne	crot_ex1
	
; copy from id + cfi buffer to the flash buffer
	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2
	lda	#$00
	sta	ptr_zp
	lda	#>tmp_id_buffer
	sta	ptr_zp+1

	ldx	#4		; 4 pages
	ldy	#0
crot_lp3:
	lda	(ptr_zp),y
	jsr	write_rbyte
	iny
	bne	crot_lp3
	inc	ptr_zp+1
	inc	rptr_zp+1
	dex
	bne	crot_lp3

	jsr	unmap_ram
; done
	lda	#<copied_msg
	ldy	#>copied_msg
	jsr	print
crot_ex1:
	jmp	sa_wait_key


dump_buffer:
	stx	tmp_zp
	jsr	print
	lda	646
	pha
	ldx	#0
	ldy	#0
db_lp1:	
	lda	(ptr2_zp),y
	jsr	print_hex			
	lda	646
	eor	#1^15
	sta	646
	iny
	inx
	dec	tmp_zp
	beq	db_ex1	
	cpx	#16
	bne	db_lp1
	ldx	#0
	jsr	print_cr
	bra	db_lp1
db_ex1:
	jsr	print_cr

	pla
	sta	646
	rts

start_msg:
	dc.b	147,"READ RAW FLASH IDENTITY...",13,13,0
id0000_msg:
	dc.b	"ID MODE ($0000):",13,0
id0100_msg:
	dc.b	"ID MODE ($0100):",13,0
cfi0000_msg:
	dc.b	"CFI QUERY ($0000):",13,0
copy_to_buffer_msg:
	dc.b	"COPY TO BUFFER? ",0
copied_msg:
	dc.b	"ID $0000-$0200, CFI $0200-$0400, OK.",13,0

;**************************************************************************
;*
;* NAME  cmd_empty_test_sector
;*
;* DESCRIPTION
;*   read sector to check if it is empty
;*
;******
cmd_empty_test_sector:
	lda	#<empty_test_msg
	ldy	#>empty_test_msg
	jsr	print

	jsr	print_sector
	jsr	print_cr
	
	jsr	map_flash

	lda	curr_sector
	jsr	set_sector_addr_end

	jsr	check_empty
	jsr	unmap_flash
	cmp	#$ff
	bne	cets_notempty

	lda	#<empty_msg
	ldy	#>empty_msg
	jsr	print
	jmp	sa_wait_key

cets_notempty:	
	lda	#<not_empty_msg
	ldy	#>not_empty_msg
	jsr	print
	jmp	sa_wait_key

empty_test_msg:
	dc.b	147,"EMPTY TEST...",13,13,0
empty_msg:
	dc.b	"EMPTY.",13,0
not_empty_msg:
	dc.b	"NOT EMPTY!",13,0



;**************************************************************************
;*
;* NAME  cmd_program_sector
;*
;* DESCRIPTION
;*   program sector from buffer
;*
;******
cmd_program_sector:
	lda	#<program_sector_msg
	ldy	#>program_sector_msg
	jsr	print

	jsr	print_sector
	jsr	print_cr

	jsr	ask_sure
	bcc	cps_ex1

	jsr	map_flash

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2

	lda	curr_sector
	jsr	sector_program

	jsr	unmap_flash
	jsr	print_err

	jmp	sa_wait_key

cps_ex1:
	jmp	sa_exit_refresh

program_sector_msg:
	dc.b	147,"PROGRAM SECTOR...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_erase_sector
;*
;* DESCRIPTION
;*   erase sector
;*
;******
cmd_erase_sector:
	lda	#<erase_sector_msg
	ldy	#>erase_sector_msg
	jsr	print

	jsr	print_sector
	jsr	print_cr
	
	jsr	ask_sure
	bcc	ces_ex1

	jsr	map_flash

	lda	curr_sector	
	jsr	sector_erase

	jsr	unmap_flash
	jsr	print_err

	jmp	sa_wait_key

ces_ex1:
	jmp	sa_exit_refresh

erase_sector_msg:
	dc.b	147,"ERASE SECTOR...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_dump_sector
;*
;* DESCRIPTION
;*   dump sector to memory
;*
;******
cmd_dump_sector:
	lda	#<dump_sector_msg
	ldy	#>dump_sector_msg
	jsr	print

	jsr	map_flash

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2

	lda	curr_sector
	jsr	sector_dump

	jsr	unmap_flash
	jsr	print_err

	jmp	sa_wait_key

dump_sector_msg:
	dc.b	147,"DUMP SECTOR...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_verify_sector
;*
;* DESCRIPTION
;*   verify sector against memory
;*
;******
cmd_verify_sector:
	lda	#<verify_sector_msg
	ldy	#>verify_sector_msg
	jsr	print

	jsr	map_flash

	lda	#0
	sta	rptr_zp
	sta	rptr_zp+1
	lda	#(BUFFER_START>>16) & $ff
	sta	rptr_zp+2

	lda	curr_sector
	jsr	sector_verify

	jsr	unmap_flash
	jsr	print_err

	jmp	sa_wait_key

verify_sector_msg:
	dc.b	147,"VERIFY SECTOR...",13,13,0

;**************************************************************************
;*
;* NAME  cmd_lockdown_sector
;*
;* DESCRIPTION
;*   set sector lockdown
;*
;******
cmd_lockdown_sector:
	lda	#<lockdown_sector_msg
	ldy	#>lockdown_sector_msg
	jsr	print

	jsr	print_sector
	jsr	print_cr

	jsr	ask_sure
	bcc	clds_ex1

	jsr	map_flash
	lda	curr_sector	
	jsr	sector_lockdown
	jsr	unmap_flash

	jsr	print_err

	jmp	sa_wait_key

clds_ex1:
	jmp	sa_exit_refresh

lockdown_sector_msg:
	dc.b	147,"LOCKDOWN SECTOR...",13,13,0

;**************************************************************************
;*
;* NAME  print_type
;*
;* DESCRIPTION
;*   Try to identify the type of flash
;*
;******
print_type:
	lda	flash_type
	asl
	tax
	lda	type_strs,x
	ldy	type_strs+1,x
	jmp	print
	
type_strs:
	dc.w	type0_str,type1_str,type2_str,type3_str,type4_str,type5_str
type0_str:
	dc.b	"NOT FOUND!",0
type1_str:
	dc.b	"UNKNOWN",0
type2_str:
	dc.b	"AT4XBV16X",0
type3_str:
	dc.b	"AT4XBV16XT",0
type4_str:
	dc.b	"SST39VF1681",0
type5_str:
	dc.b	"SST39VF1682",0
	

;**************************************************************************
;*
;* NAME  print_fptr, print_eptr, print_rptr
;*
;* DESCRIPTION
;*   print the flash pointer.
;*   
;******
print_fptr:
	lda	fptr_zp+2
	jsr	print_hex
	lda	fptr_zp+1
	jsr	print_hex
	lda	fptr_zp+0
	jmp	print_hex
print_eptr:
	lda	eptr_zp+2
	jsr	print_hex
	lda	eptr_zp+1
	jsr	print_hex
	lda	eptr_zp+0
	jmp	print_hex
print_rptr:
	lda	rptr_zp+2
	jsr	print_hex
	lda	rptr_zp+1
	jsr	print_hex
	lda	rptr_zp+0
	jmp	print_hex

;**************************************************************************
;*
;* NAME  print_sector
;*
;* DESCRIPTION
;*   print sector range.
;*   
;******
print_sector:
	lda	#"$"
	jsr	$ffd2
	lda	curr_sector
	jsr	set_sector_addr
	jsr	print_fptr
	lda	#"-"
	jsr	$ffd2
	lda	#"$"
	jsr	$ffd2
	ldx	curr_sector
	inx
	txa
	jsr	set_sector_addr
	jsr	print_fptr
	rts

; just to keep alignment identical to flash-1.0pre1.  Remove at will.
	align	4
;**************************************************************************
;*
;* SECTION  external code
;*   
;******
	include "screen_io.asm"
	include "flash_io.asm"
	
	
	seg	data
;**************************************************************************
;*
;* SECTION  data
;*   
;******
curr_sector:
	dc.b	$20		; $1f2000 on an atmel49bvxxT
	
;**************************************************************************
;*
;* DESCRIPTION
;*   miscellaneous strings.
;*   
;******
failed_msg:
	dc.b	"FAILED!  STATUS $",0
	
ok_msg:
	dc.b	"OK.",13,0
	
show_map_msg:
	dc.b	18 ; rvs on
	dc.b	"    00000000000000001111111111111111    "
	dc.b	"    0123456789ABCDEF0123456789ABCDEF    "
	dc.b	"0000",146,"                                ",18,"0000"
	dc.b	"1000",146,"                                ",18,"1000"
	dc.b	"2000",146,"                                ",18,"2000"
	dc.b	"3000",146,"                                ",18,"3000"
	dc.b	"4000",146,"                                ",18,"4000"
	dc.b	"5000",146,"                                ",18,"5000"
	dc.b	"6000",146,"                                ",18,"6000"
	dc.b	"7000",146,"                                ",18,"7000"
	dc.b	"8000",146,"                                ",18,"8000"
	dc.b	"9000",146,"                                ",18,"9000"
	dc.b	"A000",146,"                                ",18,"A000"
	dc.b	"B000",146,"                                ",18,"B000"
	dc.b	"C000",146,"                                ",18,"C000"
	dc.b	"D000",146,"                                ",18,"D000"
	dc.b	"E000",146,"                                ",18,"E000"
	dc.b	"F000",146,"                                ",18,"F000"
	dc.b	"                                        "     
	dc.b	"                                        "     
	dc.b	146 ; rvs off
	dc.b	0
	
	
	seg.u	bss
;**************************************************************************
;*
;* SECTION  bss
;*   
;******
	align	256
tmp_id_buffer:
	ds.b	512
tmp_cfi_buffer:
	ds.b	512

;**************************************************************************
;*
;* SECTION  external code
;*   
;* DESCRIPTION
;*   This is overlayed by the flash window when flashing.
;*   
;******
	seg	code2
	org	$4000
	include	"help.asm"	
helptext:
	incbin	"flashhelp_fixed.bin"
	include	"warning.asm"	
	include	"intro.asm"	
code2_end	equ	.
;**************************************************************************
;*
;* SECTION  endmarkers
;*   
;******
	seg	code
code_end	equ	.
	seg	data
data_end	equ	.
	seg.u	bss
bss_end		equ	.

	
	echo	"code_end",code_end
	echo	"data_end",data_end
	echo	"bss_end",bss_end
	echo	"code2_end",code2_end
; eof
